const Koa = require("koa");
const Router = require("koa-router");
const bodyParser = require("koa-bodyparser");
const compose = require("koa-compose");
const dotenv = require("dotenv");
const conf = require("./conf");
const pgos = require("./pgos_lib");

dotenv.config();

const app = new Koa();
const router = new Router();

router.get("/", async (ctx) => {
  ctx.body = "Hello World!";
});

// client_hello invoke respond a simple 'Hello' to the invoker, ONLY for client side invoking
router.post("/client_hello", async (ctx) => {
  var invoker = ctx.request.headers["invoker"];
  if (invoker != "client") {
    return;
  }
  var name = ctx.request.rawBody;
  ctx.headers["content-type"] = "text/plain";
  ctx.body = `Hello client ${name}!`;
});

// backend_hello invoke respond a simple 'Hello' to the invoker, ONLY for backend side invoking(by backend http api)
router.post("/backend_hello", async (ctx) => {
  var invoker = ctx.request.headers["invoker"];
  if (invoker != "backend") {
    return;
  }
  var name = ctx.request.rawBody;
  ctx.headers["content-type"] = "text/plain";
  ctx.body = `Hello backend ${name}!`;
});

// backend_get_player_info is a sample uri to demostrate backend side invoking
// it invokes backend http api to get a player's info, and respond to the invoker, ONLY for backend side invoking(by backend http api)
// you can invoke any http api based on your demand, even call other virtual server :-)
router.post("/backend_get_player_info", async (ctx) => {
  var invoker = ctx.request.headers["invoker"];
  if (invoker != "backend") {
    return;
  }
  var req_body = ctx.request.body;
  console.log(ctx.request.body.player_id);
  var request = { player_id: ctx.request.body.player_id };
  var ret = await pgos.invokeBackendApi(
    ctx.request.headers["titleid"],
    ctx.request.headers["titleregionid"],
    conf.serverKey,
    "/player_svr/get_player_info",
    JSON.stringify(request)
  );
  if (ret == null) {
    ctx.status = 403;
    ctx.headers["content-type"] = "text/plain";
    ctx.body = "invalid server key";
    return;
  }
  console.log("%s, response body: %s", ctx.URL, ret.data);
  ctx.headers["content-type"] = "application/json";
  ctx.body = ret.data;
});

// event handle all event, please config the event handler in the portal in advance
router.post("/pgos_event", async (ctx) => {
  var invoker = ctx.request.headers["invoker"];
  if (invoker != "event") {
    ctx.status = 401;
    return;
  }
  // write your own code to handle
  console.log("/pgos_event: " + ctx.request.rawBody);
  ctx.body = "";
});

// pgos_economy handles the specific requests from PGOS economy service, please config the binding for the economy service on the portal in advance
router.post("/pgos_economy", async (ctx) => {
  var invoker = ctx.request.headers["invoker"];
  if (invoker != "economy") {
    ctx.status = 401;
    return;
  }
  // write your own code to handle
  console.log("/pgos_economy: " + ctx.request.rawBody);
  ctx.body = "";
});

// pgos_task handles the task request, please config the task on the portal in advance
router.post("/pgos_task", async (ctx) => {
  var invoker = ctx.request.headers["invoker"];
  if (invoker != "task") {
    ctx.status = 401;
    return;
  }
  // write your own code to handle
  console.log("/pgos_task: " + ctx.request.rawBody);
  ctx.body = "";
});

const middleware = compose([
  bodyParser(),
  router.routes(),
  router.allowedMethods(),
]);

app.use(middleware);

app.listen(process.env.PORT || 8081);
