// a lib to help you invoking PGOS backend Http API
const crypto = require("crypto");
const axios = require("axios");
const { Buffer } = require("buffer");
const ServerTokenIv = Buffer.from("$3,.'/&^rgnjkl!#", "utf-8");
class AesCbc {
  constructor(key, iv) {
    this.key = key;
    this.iv = iv;
  }
  encrypt(text) {
    const cipher = crypto.createCipheriv("aes-128-cbc", this.key, this.iv);
    const paddedText = this.pad(Buffer.from(text, "utf-8"));
    const encrypted = Buffer.concat([
      cipher.update(paddedText),
      cipher.final(),
    ]);
    return encrypted.toString("base64");
  }
  decrypt(text) {
    const decipher = crypto.createDecipheriv("aes-128-cbc", this.key, this.iv);
    const decodedText = Buffer.from(text, "base64");
    const decrypted = Buffer.concat([
      decipher.update(decodedText),
      decipher.final(),
    ]);
    return this.unPad(decrypted).toString("utf-8");
  }
  pad(text) {
    const blockSize = 16;
    const padding = blockSize - (text.length % blockSize);
    const padText = Buffer.alloc(padding, padding);
    return Buffer.concat([text, padText]);
  }
  unPad(text) {
    const unPadding = text[text.length - 1];
    return text.slice(0, text.length - unPadding);
  }
}

function encryptServerTicket(key, ticketJsonStr) {
  const ac = new AesCbc(Buffer.from(key, "utf-8"), ServerTokenIv);
  return ac.encrypt(ticketJsonStr);
}

function decryptServerTicket(key, ticketJsonStr) {
  const ac = new AesCbc(Buffer.from(key, "utf-8"), ServerTokenIv);
  return ac.decrypt(ticketJsonStr);
}

class SecretTicket {
  constructor(titleRegionID, secretID, time) {
    this.title_region_id = titleRegionID;
    this.secret_id = secretID;
    this.time = time;
  }
}

function genServerTicket(secretID, secretKey, titleRegionID) {
  const st = new SecretTicket(
    titleRegionID,
    secretID,
    Math.floor(Date.now() / 1000)
  );
  const ticketJson = JSON.stringify(st);
  const key = secretKey.split("-").join("");
  const ticket = encryptServerTicket(key, ticketJson);
  return ticket;
}

function splitServerKey(serverKey) {
  var s = serverKey.split("-");
  if (s.length > 1) {
    secretID = s[0];
    secretKey = s.slice(1).join("-");
    return { secretID: secretID, secretKey: secretKey };
  }
  return null;
}

function getServerUrl(titleRegionID) {
  let location = "euff";
  let domainSuffix = "com";
  let arr = titleRegionID.split("_");
  if (arr.length > 0) {
    location = arr[0];
  }
  if (location === "t" || location === "d" || location.indexOf("cn") === 0) {
    domainSuffix = "cn";
  }
  let serverUrl = `https://${location}.server.pgos.intlgame.${domainSuffix}`;
  return serverUrl;
}

async function invokeBackendApi(
  titleID,
  titleRegionID,
  serverKey,
  apiUri,
  bodyStr
) {
  var k = splitServerKey(serverKey);
  if (k == null) {
    return null;
  }
  var ticket = genServerTicket(k.secretID, k.secretKey, titleRegionID);
  let url = getServerUrl(titleRegionID) + apiUri;
  let headers = {
    "Content-Type": "application/json",
    TitleId: titleID,
    TitleRegionId: titleRegionID,
    SecretId: k.secretID,
    ServerTicket: ticket,
  };
  //console.log(`Request ${url}:\nHeaders:`);
  //console.log(headers);
  console.log(`Body:\n${bodyStr}\n---------------------------------`);
  try {
    let response = await axios.post(url, bodyStr, { headers, timeout: 3000 });
    //console.log(`Response ${response.status}\nHeaders:`);
    //console.log(response.headers);
    //console.log(`Body:\n${JSON.stringify(response.data)}\n`);
    return { headers: response.headers, data: response.data };
  } catch (err) {
    //console.error(`Error: ${err}`);
    return { err: err };
  }
}

module.exports = { genServerTicket, getServerUrl, invokeBackendApi };
